#include "XC2dVTIWaveprop.c"

// dimensional information ( t,x,y,z,s, note that x,y,z info are after tapering/padding)
int   XC2dVTIMod_nt,XC2dVTIMod_nx,XC2dVTIMod_nz,XC2dVTIMod_ns;
float XC2dVTIMod_ot,XC2dVTIMod_ox,XC2dVTIMod_oz;
float XC2dVTIMod_dt,XC2dVTIMod_dx,XC2dVTIMod_dz;
float *XC2dVTIMod_sdep, *XC2dVTIMod_rdep;
int *XC2dVTIMod_isdep, *XC2dVTIMod_irdep;
//tapering/padding information ( in x,y,z directions) 
int   XC2dVTIMod_nxtape,XC2dVTIMod_nztape;
int   XC2dVTIMod_nxt,XC2dVTIMod_nzt;
float *XC2dVTIMod_vel;
float *XC2dVTIMod_veldtinvdx2;
float *XC2dVTIMod_epsf;
float *XC2dVTIMod_deltaf;
float XC2dVTIMod_vmax;
//incremental z range of wavefield to be calculated as t increases
int   XC2dVTIMod_izmax_inc;

#define zxloc(iz,ix) ((iz)+(ix)*XC2dVTIMod_nz)
#define Errmsg(str)  {fprintf(stderr,str);return 1;}
#define Errmsg2(str,str1)  {fprintf(stderr,str,str1);return 1;}
#define Errmsg3(str,str1,str2)  {fprintf(stderr,str,str1,str2);return 1;}

int XC2dVTIMod_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \
                      int nxtapein, int nztapein,             \
                      float* sdepin, float *rdepin, float* velin, float *epsin, float *deltain, \
                      int needwvmvin, int jtmvin );
int XC2dVTIMod_op(int tsign, int is, float *src, float *pdat, float *rdat, float *wvmv);
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIMod_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \ 
                      int nxtapein, int nztapein,             \ 
                      float* sdepin, float *rdepin, float* velin, float *epsin, float *deltain, \
                      int needwvmvin, int jtmvin )
{
    int ix,iy,iz,stat;
    float temp,v,vmax;
    float xval;
    // take input parameters
    XC2dVTIMod_nt=ntin;
    XC2dVTIMod_nx=nxin;
    XC2dVTIMod_nz=nzin;
    XC2dVTIMod_ns=nsin;

    XC2dVTIMod_ot=otin;
    XC2dVTIMod_ox=oxin;
    XC2dVTIMod_oz=ozin;

    XC2dVTIMod_dt=dtin;
    XC2dVTIMod_dx=dxin;
    XC2dVTIMod_dz=dzin;

    XC2dVTIMod_nxtape=nxtapein;
    XC2dVTIMod_nztape=nztapein;

    XC2dVTIMod_sdep=sdepin;
    XC2dVTIMod_rdep=rdepin;
    XC2dVTIMod_vel=velin;

    // determine if padded region is reasonable
    if (nxin<=2*nxtapein)
       WpErrmsg("Error, nx less or equal than twice of padded x reigon\n")
    if (nzin<=2*nztapein)
       WpErrmsg("Error, nz less or equal than twice of padded z reigon\n")

    // determine if dx==dz
    if (  XC2dVTIMod_dz!=XC2dVTIMod_dx )
       Errmsg("ERROR, dz not equal to dx, exit!!!\n")
    else 
    {
       // produce (vel*dt/dx)**2 and get vmax/izmax_inc on the way
       vmax=0.;
       XC2dVTIMod_isdep=(int*)malloc(nxin*sizeof(int));
       XC2dVTIMod_irdep=(int*)malloc(nxin*sizeof(int));
       XC2dVTIMod_veldtinvdx2=(float*)malloc(nxin*nzin*sizeof(float));
       XC2dVTIMod_epsf=(float*)malloc(nxin*nzin*sizeof(float));
       XC2dVTIMod_deltaf=(float*)malloc(nxin*nzin*sizeof(float));
       xval=0;
       for (ix=0;ix<nxin;ix++)
          for (iz=0;iz<nzin;iz++)
          {
             v=XC2dVTIMod_vel[ zxloc(iz,ix) ];
             temp= v * XC2dVTIMod_dt/XC2dVTIMod_dx;
             XC2dVTIMod_veldtinvdx2[ zxloc(iz,ix) ]=temp*temp;
             if ( v  > vmax)
                vmax = v;
             if (temp>0.86)
                Errmsg2("ERROR, stability condition for wavepropagation is violated, temp=%f which is larger than 0.86\n",temp)
             XC2dVTIMod_epsf[ zxloc(iz,ix) ]=temp*temp*(1.+2*epsin[zxloc(iz,ix)]);
             XC2dVTIMod_deltaf[ zxloc(iz,ix) ]=temp*temp*sqrt((1.+2*deltain[zxloc(iz,ix)]));
             xval+=temp;
          }
       fprintf(stderr,"vmax=%e,xval=%e\n",vmax,xval);
       XC2dVTIMod_izmax_inc= vmax*XC2dVTIMod_dt/XC2dVTIMod_dz + 1;
    }
    // initilize wave propagation module
    int freesurf=0;
    long memcnt=0; 
    stat=XC2dVTIWaveprop_Init(ntin,nxin,nzin,nxtapein,nztapein,needwvmvin,jtmvin, freesurf, &memcnt);
    if (stat==1)
       Errmsg("Error in XC2dVTIWaveprop_Init, program terminated!\n")

    return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIMod_op(int tsign, int is, float *src, float *pdat, float *rdat, float *wvmv)
{
  float *sdep,*rdep, *dummywvmv;
  int ix;
  int nx;
  int izmax_inc,stat;
  float dz;

  //VerbMsg("Calculating sdep\n")
  sdep=&XC2dVTIMod_sdep[is*XC2dVTIMod_nx];
  rdep=&XC2dVTIMod_rdep[is*XC2dVTIMod_nx];
 
  nx=XC2dVTIMod_nx;
  dz=XC2dVTIMod_dz;
  // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
  //VerbMsg("Calculating isdep\n")
  for (ix=0;ix<nx;ix++)
     XC2dVTIMod_isdep[ ix ]=(sdep[ ix ]-XC2dVTIMod_oz)/dz+XC2dVTIMod_nztape;

  //VerbMsg("Calculating irdep\n")
  for (ix=0;ix<nx;ix++)
     XC2dVTIMod_irdep[ ix ]=(rdep[ ix ]-XC2dVTIMod_oz)/dz+XC2dVTIMod_nztape;


  // wave propgation
  izmax_inc=XC2dVTIMod_izmax_inc;
  stat=XC2dVTIWaveprop_op( tsign, XC2dVTIMod_isdep, XC2dVTIMod_irdep, src, XC2dVTIMod_veldtinvdx2, XC2dVTIMod_epsf, XC2dVTIMod_deltaf, izmax_inc, pdat, rdat, dummywvmv, wvmv );

  if (stat==1) 
     Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  
  return 0;
}
