#include "XC2dVTIWaveprop.c"

// dimensional information ( t,x,y,z,s, note that x,y,z info are after tapering/padding)
int   XC2dVTIRTM_nt,XC2dVTIRTM_nx,XC2dVTIRTM_nz,XC2dVTIRTM_ns;
float XC2dVTIRTM_ot,XC2dVTIRTM_ox,XC2dVTIRTM_oz;
float XC2dVTIRTM_dt,XC2dVTIRTM_dx,XC2dVTIRTM_dz;
float *XC2dVTIRTM_sdep, *XC2dVTIRTM_rdep;
int *XC2dVTIRTM_isdep, *XC2dVTIRTM_irdep;
//tapering/padding information ( in x,y,z directions) 
int   XC2dVTIRTM_nxtape,XC2dVTIRTM_nztape;
int   XC2dVTIRTM_jtmv;
int   XC2dVTIRTM_nxt,XC2dVTIRTM_nzt;
float *XC2dVTIRTM_pwvmv1, *XC2dVTIRTM_pwvmv2;
float *XC2dVTIRTM_rwvmv1, *XC2dVTIRTM_rwvmv2;
float *XC2dVTIRTM_vel;
float *XC2dVTIRTM_veldtinvdx2;
float *XC2dVTIRTM_epsf;
float *XC2dVTIRTM_deltaf;
float XC2dVTIRTM_vmax;
//incremental z range of wavefield to be calculated as t increases
int   XC2dVTIRTM_izmax_inc;

#define zxloc(iz,ix) ((iz)+(ix)*XC2dVTIRTM_nz)
#define Errmsg(str)  {fprintf(stderr,str);return 1;}
#define Errmsg2(str,str1)  {fprintf(stderr,str,str1);return 1;}
#define Errmsg3(str,str1,str2)  {fprintf(stderr,str,str1,str2);return 1;}

int XC2dVTIRTM_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \
                      int nxtapein, int nztapein,             \
                      float* sdepin, float *rdepin, float* velin, float *epsin, float *deltain, \
                      int jtmvin );
int XC2dVTIRTM_op(float *src, float *dat, float *imag);
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIRTM_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \ 
                      int nxtapein, int nztapein,             \ 
                      float* sdepin, float *rdepin, float* velin, float *epsin, float *deltain, \
                      int jtmvin )
{
    int ix,iy,iz,stat;
    float temp,v,vmax;
    float xval;
    // take input parameters
    XC2dVTIRTM_nt=ntin;
    XC2dVTIRTM_nx=nxin;
    XC2dVTIRTM_nz=nzin;
    XC2dVTIRTM_ns=nsin;

    XC2dVTIRTM_ot=otin;
    XC2dVTIRTM_ox=oxin;
    XC2dVTIRTM_oz=ozin;

    XC2dVTIRTM_dt=dtin;
    XC2dVTIRTM_dx=dxin;
    XC2dVTIRTM_dz=dzin;

    XC2dVTIRTM_nxtape=nxtapein;
    XC2dVTIRTM_nztape=nztapein;
    XC2dVTIRTM_jtmv=jtmvin;

    XC2dVTIRTM_sdep=sdepin;
    XC2dVTIRTM_rdep=rdepin;
    XC2dVTIRTM_vel=velin;

    // determine if padded region is reasonable
    if (nxin<=2*nxtapein)
       WpErrmsg("Error, nx less or equal than twice of padded x reigon\n")
    if (nzin<=2*nztapein)
       WpErrmsg("Error, nz less or equal than twice of padded z reigon\n")

    // determine if dx==dz
    if (  XC2dVTIRTM_dz!=XC2dVTIRTM_dx )
       Errmsg("ERROR, dz not equal to dx, exit!!!\n")
    else 
    {
       // produce (vel*dt/dx)**2 and get vmax/izmax_inc on the way
       vmax=0.;
       XC2dVTIRTM_isdep=(int*)malloc(nxin*sizeof(int));
       XC2dVTIRTM_irdep=(int*)malloc(nxin*sizeof(int));
       XC2dVTIRTM_veldtinvdx2=(float*)malloc(nxin*nzin*sizeof(float));
       XC2dVTIRTM_epsf=(float*)malloc(nxin*nzin*sizeof(float));
       XC2dVTIRTM_deltaf=(float*)malloc(nxin*nzin*sizeof(float));
       XC2dVTIRTM_rwvmv1=(float*)malloc(nxin*nzin*(ntin/jtmvin+1)*sizeof(float));
       XC2dVTIRTM_rwvmv2=(float*)malloc(nxin*nzin*(ntin/jtmvin+1)*sizeof(float));
       xval=0;
       for (ix=0;ix<nxin;ix++)
          for (iz=0;iz<nzin;iz++)
          {
             v=XC2dVTIRTM_vel[ zxloc(iz,ix) ];
             temp= v * XC2dVTIRTM_dt/XC2dVTIRTM_dx;
             XC2dVTIRTM_veldtinvdx2[ zxloc(iz,ix) ]=temp*temp;
             if ( v  > vmax)
                vmax = v;
             if (temp>0.86)
                Errmsg2("ERROR, stability condition for wavepropagation is violated, temp=%f which is larger than 0.86\n",temp)
             XC2dVTIRTM_epsf[ zxloc(iz,ix) ]=temp*temp*(1.+2*epsin[zxloc(iz,ix)]);
             XC2dVTIRTM_deltaf[ zxloc(iz,ix) ]=temp*temp*sqrt((1.+2*deltain[zxloc(iz,ix)]));
             xval+=temp;
          }
       fprintf(stderr,"vmax=%e,xval=%e\n",vmax,xval);
       XC2dVTIRTM_izmax_inc= vmax*XC2dVTIRTM_dt/XC2dVTIRTM_dz + 1;
    }
    // initilize wave propagation module
    int freesurf=0;
    long memcnt=0; 
    int needwvmvin=1;
    //by doing this, will only enable recording of rwvmv ( vertical stress)
    stat=XC2dVTIWaveprop_Init(ntin,nxin,nzin,nxtapein,nztapein,needwvmvin,jtmvin, freesurf, &memcnt);
    if (stat==1)
       Errmsg("Error in XC2dVTIWaveprop_Init, program terminated!\n")

    return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIRTM_op(float *src, float *dat, float *imag)
{
  float *sdep,*rdep, *pwvmv1, *pwvmv2, *rwvmv1, *rwvmv2, *dummydat;
  int ix,is,it,i;
  int nx,nz,ns,nt;
  int izmax_inc,stat,jtmv;
  float dz;
  int tsign;
  ns=XC2dVTIRTM_ns;
  nx=XC2dVTIRTM_nx;
  nz=XC2dVTIRTM_nz;
  nt=XC2dVTIRTM_nt;
  dz=XC2dVTIRTM_dz;
  rwvmv1=XC2dVTIRTM_rwvmv1;
  rwvmv2=XC2dVTIRTM_rwvmv2;
  pwvmv1=XC2dVTIRTM_pwvmv1;
  pwvmv2=XC2dVTIRTM_pwvmv2;
  jtmv=XC2dVTIRTM_jtmv;
  int needdat=0;
  XC2dVTIWaveprop_setrcddat(needdat);
  for (i=0;i<nx*nz;i++)
     imag[i]=0;
  for (is=0;is<ns;is++)
  {
  //VerbMsg("Calculating sdep\n")
     sdep=&XC2dVTIRTM_sdep[is*XC2dVTIRTM_nx];
     rdep=&XC2dVTIRTM_rdep[is*XC2dVTIRTM_nx];
 
     // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_isdep[ ix ]=(sdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_irdep[ ix ]=(rdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;


     // wave propgation
     //only recording/imaging rwvmv ( vertical stress)
     izmax_inc=XC2dVTIRTM_izmax_inc;
     tsign=1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_isdep, XC2dVTIRTM_irdep, &src[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv1, rwvmv1 );

     tsign=-1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_irdep, XC2dVTIRTM_isdep, &dat[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv2, rwvmv2 );

    for (it=0;it<nt/jtmv;it++)
       for (i=0;i<nx*nz;i++)
          imag[i]+=rwvmv1[i+nx*nz*it]*rwvmv2[i+nx*nz*it];
 
     if (stat==1) 
        Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  }  
  
  return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//VTI aniso sensitivity kernel with variables being (1+2eps) and (1/vp)^2 /////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIRTM_wvkernel1(float *src, float *dat, float *imag)
{
  float *sdep,*rdep, *pwvmv1, *pwvmv2, *rwvmv1, *rwvmv2, *dummydat;
  int ix,is,iz,it,i,idx;
  int nx,nz,ns,nt;
  int izmax_inc,stat,jtmv;
  float dz,tmp,dx,dt;
  int tsign;
  ns=XC2dVTIRTM_ns;
  nx=XC2dVTIRTM_nx;
  nz=XC2dVTIRTM_nz;
  nt=XC2dVTIRTM_nt;
  dz=XC2dVTIRTM_dz;
  jtmv=XC2dVTIRTM_jtmv;
  dt=XC2dVTIRTM_dt;
  dx=XC2dVTIRTM_dx;
  XC2dVTIRTM_pwvmv1=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  XC2dVTIRTM_pwvmv2=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  rwvmv1=XC2dVTIRTM_rwvmv1;
  rwvmv2=XC2dVTIRTM_rwvmv2;
  pwvmv1=XC2dVTIRTM_pwvmv1;
  pwvmv2=XC2dVTIRTM_pwvmv2;
  int needdat=0;
  XC2dVTIWaveprop_setrcddat(needdat);
  for (i=0;i<nx*nz*4;i++)
     imag[i]=0;
  
  int needwvmv=1;
  XC2dVTIWaveprop_setpwvmv(needwvmv);
  XC2dVTIWaveprop_setrwvmv(needwvmv);
  for (is=0;is<ns;is++)
  {
  //VerbMsg("Calculating sdep\n")
     sdep=&XC2dVTIRTM_sdep[is*XC2dVTIRTM_nx];
     rdep=&XC2dVTIRTM_rdep[is*XC2dVTIRTM_nx];
 
     // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_isdep[ ix ]=(sdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_irdep[ ix ]=(rdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for(i=0;i<nx*nz*(nt/jtmv+1);i++)
     {
        pwvmv1[i]=0;
        pwvmv2[i]=0;
        rwvmv1[i]=0;
        rwvmv2[i]=0;
     }
     // wave propgation
     //only recording/imaging rwvmv ( vertical stress)
     izmax_inc=XC2dVTIRTM_izmax_inc;
     tsign=1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_isdep, XC2dVTIRTM_irdep, &src[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv1, rwvmv1 );

     tsign=-1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_irdep, XC2dVTIRTM_isdep, &dat[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv2, rwvmv2 );

    for (it=2;it<nt/jtmv-2;it++)
       for (ix=2;ix<nx-2;ix++)
          for (iz=0;iz<nz;iz++)
          {
             idx=Wpzxloc(iz,ix);
             tmp=-1./12.*(pwvmv1[idx-2*nz+it*nx*nz]+pwvmv1[idx+2*nz+it*nx*nz])+4./3.*(pwvmv1[idx-nz+it*nx*nz]+pwvmv1[idx+nz+it*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             imag[idx]+=tmp*pwvmv2[idx+nx*nz*it];
             tmp=-1./12.*(pwvmv1[idx+(it-2)*nx*nz]+pwvmv1[idx+(it+2)*nx*nz])+4./3.*(pwvmv1[idx+(it-1)*nx*nz]+pwvmv1[idx+(it+1)*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             imag[idx+2*nx*nz]+=tmp*pwvmv2[idx+nx*nz*it];
             tmp=-1./12.*(rwvmv1[idx+(it-2)*nx*nz]+rwvmv1[idx+(it+2)*nx*nz])+4./3.*(rwvmv1[idx+(it-1)*nx*nz]+rwvmv1[idx+(it+1)*nx*nz])-2.5*rwvmv1[idx+it*nx*nz];
             imag[idx+3*nx*nz]+=tmp*rwvmv2[idx+nx*nz*it];
             //imag[idx+3*nx*nz]+=rwvmv1[idx+nx*nz*it]*rwvmv2[idx+nx*nz*it];
          } 
    if (stat==1) 
        Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  }  
  
  for (ix=2;ix<nx-2;ix++)
     for (iz=0;iz<nz;iz++)
     {
        idx=Wpzxloc(iz,ix);
        imag[idx]=imag[idx]/(dx*dx);
        imag[idx+2*nx*nz]=imag[idx+2*nx*nz]/(dt*dt);
        imag[idx+3*nx*nz]=imag[idx+3*nx*nz]/(dt*dt);
     }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//VTI aniso sensitivity kernel with variables being (1+2eps) and 40.*log((1/vp)^2) /////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dVTIRTM_wvkernel11(float *src, float *dat, float *imag)
{
  float *sdep,*rdep, *pwvmv1, *pwvmv2, *rwvmv1, *rwvmv2, *dummydat;
  int ix,is,iz,it,i,idx;
  int nx,nz,ns,nt;
  int izmax_inc,stat,jtmv;
  float dz,tmp,dx,dt;
  int tsign;
  ns=XC2dVTIRTM_ns;
  nx=XC2dVTIRTM_nx;
  nz=XC2dVTIRTM_nz;
  nt=XC2dVTIRTM_nt;
  dz=XC2dVTIRTM_dz;
  jtmv=XC2dVTIRTM_jtmv;
  dt=XC2dVTIRTM_dt;
  dx=XC2dVTIRTM_dx;
  XC2dVTIRTM_pwvmv1=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  XC2dVTIRTM_pwvmv2=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  rwvmv1=XC2dVTIRTM_rwvmv1;
  rwvmv2=XC2dVTIRTM_rwvmv2;
  pwvmv1=XC2dVTIRTM_pwvmv1;
  pwvmv2=XC2dVTIRTM_pwvmv2;
  int needdat=0;
  XC2dVTIWaveprop_setrcddat(needdat);
  for (i=0;i<nx*nz*4;i++)
     imag[i]=0;
  
  int needwvmv=1;
  XC2dVTIWaveprop_setpwvmv(needwvmv);
  XC2dVTIWaveprop_setrwvmv(needwvmv);
  for (is=0;is<ns;is++)
  {
  //VerbMsg("Calculating sdep\n")
     sdep=&XC2dVTIRTM_sdep[is*XC2dVTIRTM_nx];
     rdep=&XC2dVTIRTM_rdep[is*XC2dVTIRTM_nx];
 
     // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_isdep[ ix ]=(sdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_irdep[ ix ]=(rdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for(i=0;i<nx*nz*(nt/jtmv+1);i++)
     {
        pwvmv1[i]=0;
        pwvmv2[i]=0;
        rwvmv1[i]=0;
        rwvmv2[i]=0;
     }
     // wave propgation
     //only recording/imaging rwvmv ( vertical stress)
     izmax_inc=XC2dVTIRTM_izmax_inc;
     tsign=1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_isdep, XC2dVTIRTM_irdep, &src[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv1, rwvmv1 );

     tsign=-1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_irdep, XC2dVTIRTM_isdep, &dat[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv2, rwvmv2 );

    for (it=2;it<nt/jtmv-2;it++)
       for (ix=2;ix<nx-2;ix++)
          for (iz=0;iz<nz;iz++)
          {
             idx=Wpzxloc(iz,ix);
             tmp=-1./12.*(pwvmv1[idx-2*nz+it*nx*nz]+pwvmv1[idx+2*nz+it*nx*nz])+4./3.*(pwvmv1[idx-nz+it*nx*nz]+pwvmv1[idx+nz+it*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             imag[idx]+=tmp*pwvmv2[idx+nx*nz*it];
             tmp=-1./12.*(pwvmv1[idx+(it-2)*nx*nz]+pwvmv1[idx+(it+2)*nx*nz])+4./3.*(pwvmv1[idx+(it-1)*nx*nz]+pwvmv1[idx+(it+1)*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             imag[idx+2*nx*nz]+=tmp*pwvmv2[idx+nx*nz*it];
             tmp=-1./12.*(rwvmv1[idx+(it-2)*nx*nz]+rwvmv1[idx+(it+2)*nx*nz])+4./3.*(rwvmv1[idx+(it-1)*nx*nz]+rwvmv1[idx+(it+1)*nx*nz])-2.5*rwvmv1[idx+it*nx*nz];
             imag[idx+3*nx*nz]+=tmp*rwvmv2[idx+nx*nz*it];
             //imag[idx+3*nx*nz]+=rwvmv1[idx+nx*nz*it]*rwvmv2[idx+nx*nz*it];
          } 
    if (stat==1) 
        Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  }  
  
  for (ix=2;ix<nx-2;ix++)
     for (iz=0;iz<nz;iz++)
     {
        idx=Wpzxloc(iz,ix);
        imag[idx]=imag[idx]/(dx*dx);
        imag[idx+2*nx*nz]=imag[idx+2*nx*nz]/(dx*dx)/XC2dVTIRTM_veldtinvdx2[idx];
        imag[idx+3*nx*nz]=imag[idx+3*nx*nz]/(dx*dx)/XC2dVTIRTM_veldtinvdx2[idx];
        //imag[idx+2*nx*nz]=imag[idx+2*nx*nz]/(dx*dx)/XC2dVTIRTM_veldtinvdx2[idx]*0.025;
        //imag[idx+3*nx*nz]=imag[idx+3*nx*nz]/(dx*dx)/XC2dVTIRTM_veldtinvdx2[idx]*0.025;
     }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//VTI aniso sensitivity kernel with variables being (vh)^2 and vp^2 ///////////////////
int XC2dVTIRTM_wvkernel2(float *src, float *dat, float *imag)
{
  float *sdep,*rdep, *pwvmv1, *pwvmv2, *rwvmv1, *rwvmv2, *dummydat;
  int ix,is,iz,it,i,idx;
  int nx,nz,ns,nt;
  int izmax_inc,stat,jtmv;
  float dz,tmp,tmp1,dx,dt;
  int tsign;
  ns=XC2dVTIRTM_ns;
  nx=XC2dVTIRTM_nx;
  nz=XC2dVTIRTM_nz;
  nt=XC2dVTIRTM_nt;
  dz=XC2dVTIRTM_dz;
  jtmv=XC2dVTIRTM_jtmv;
  dt=XC2dVTIRTM_dt;
  dx=XC2dVTIRTM_dx;
  XC2dVTIRTM_pwvmv1=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  XC2dVTIRTM_pwvmv2=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  rwvmv1=XC2dVTIRTM_rwvmv1;
  rwvmv2=XC2dVTIRTM_rwvmv2;
  pwvmv1=XC2dVTIRTM_pwvmv1;
  pwvmv2=XC2dVTIRTM_pwvmv2;
  int needdat=0;
  XC2dVTIWaveprop_setrcddat(needdat);
  for (i=0;i<nx*nz*4;i++)
     imag[i]=0;
  
  int needwvmv=1;
  XC2dVTIWaveprop_setpwvmv(needwvmv);
  XC2dVTIWaveprop_setrwvmv(needwvmv);
  for (is=0;is<ns;is++)
  {
  //VerbMsg("Calculating sdep\n")
     sdep=&XC2dVTIRTM_sdep[is*XC2dVTIRTM_nx];
     rdep=&XC2dVTIRTM_rdep[is*XC2dVTIRTM_nx];
 
     // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_isdep[ ix ]=(sdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_irdep[ ix ]=(rdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for(i=0;i<nx*nz*(nt/jtmv+1);i++)
     {
        pwvmv1[i]=0;
        pwvmv2[i]=0;
        rwvmv1[i]=0;
        rwvmv2[i]=0;
     }
     // wave propgation
     //only recording/imaging rwvmv ( vertical stress)
     izmax_inc=XC2dVTIRTM_izmax_inc;
     tsign=1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_isdep, XC2dVTIRTM_irdep, &src[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv1, rwvmv1 );

     tsign=-1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_irdep, XC2dVTIRTM_isdep, &dat[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv2, rwvmv2 );

    for (it=0;it<nt/jtmv;it++)
       for (ix=2;ix<nx-2;ix++)
          for (iz=2;iz<nz-2;iz++)
          {
             idx=Wpzxloc(iz,ix);
             tmp=-1./12.*(pwvmv1[idx-2*nz+it*nx*nz]+pwvmv1[idx+2*nz+it*nx*nz])+4./3.*(pwvmv1[idx-nz+it*nx*nz]+pwvmv1[idx+nz+it*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             tmp1=-1./12.*(rwvmv1[idx-2+it*nx*nz]+rwvmv1[idx+2+it*nx*nz])+4./3.*(rwvmv1[idx-1+it*nx*nz]+rwvmv1[idx+1+it*nx*nz])-2.5*rwvmv1[idx+it*nx*nz];
             imag[idx]+=tmp*pwvmv2[idx+nx*nz*it];
             imag[idx+2*nx*nz]+=tmp1*pwvmv2[idx+nx*nz*it]*XC2dVTIRTM_deltaf[idx]/XC2dVTIRTM_veldtinvdx2[idx];
             imag[idx+3*nx*nz]+=tmp*rwvmv2[idx+nx*nz*it]*XC2dVTIRTM_deltaf[idx]/XC2dVTIRTM_veldtinvdx2[idx]+tmp1*rwvmv2[idx+nx*nz*it];
          } 
    if (stat==1) 
        Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  }  
  
  for (ix=2;ix<nx-2;ix++)
     for (iz=0;iz<nz;iz++)
     {
        idx=Wpzxloc(iz,ix);
        imag[idx]=imag[idx]/(dx*dx);
        imag[idx+2*nx*nz]=imag[idx+2*nx*nz]/(dx*dx);
        imag[idx+3*nx*nz]=imag[idx+3*nx*nz]/(dx*dx);
     }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//VTI aniso sensitivity kernel with variables being (vh)^2, vp*vnmo and vp^2 ///////////////////
int XC2dVTIRTM_wvkernel3(float *src, float *dat, float *imag)
{
  float *sdep,*rdep, *pwvmv1, *pwvmv2, *rwvmv1, *rwvmv2, *dummydat;
  int ix,is,iz,it,i,idx;
  int nx,nz,ns,nt;
  int izmax_inc,stat,jtmv;
  float dz,tmp,tmp1,dx,dt;
  int tsign;
  ns=XC2dVTIRTM_ns;
  nx=XC2dVTIRTM_nx;
  nz=XC2dVTIRTM_nz;
  nt=XC2dVTIRTM_nt;
  dz=XC2dVTIRTM_dz;
  jtmv=XC2dVTIRTM_jtmv;
  dt=XC2dVTIRTM_dt;
  dx=XC2dVTIRTM_dx;
  XC2dVTIRTM_pwvmv1=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  XC2dVTIRTM_pwvmv2=(float*)malloc(nx*nz*(nt/jtmv+1)*sizeof(float));
  rwvmv1=XC2dVTIRTM_rwvmv1;
  rwvmv2=XC2dVTIRTM_rwvmv2;
  pwvmv1=XC2dVTIRTM_pwvmv1;
  pwvmv2=XC2dVTIRTM_pwvmv2;
  int needdat=0;
  XC2dVTIWaveprop_setrcddat(needdat);
  for (i=0;i<nx*nz*4;i++)
     imag[i]=0;
  
  int needwvmv=1;
  XC2dVTIWaveprop_setpwvmv(needwvmv);
  XC2dVTIWaveprop_setrwvmv(needwvmv);
  for (is=0;is<ns;is++)
  {
  //VerbMsg("Calculating sdep\n")
     sdep=&XC2dVTIRTM_sdep[is*XC2dVTIRTM_nx];
     rdep=&XC2dVTIRTM_rdep[is*XC2dVTIRTM_nx];
 
     // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_isdep[ ix ]=(sdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for (ix=0;ix<nx;ix++)
        XC2dVTIRTM_irdep[ ix ]=(rdep[ ix ]-XC2dVTIRTM_oz)/dz+XC2dVTIRTM_nztape;

     for(i=0;i<nx*nz*(nt/jtmv+1);i++)
     {
        pwvmv1[i]=0;
        pwvmv2[i]=0;
        rwvmv1[i]=0;
        rwvmv2[i]=0;
     }
     // wave propgation
     //only recording/imaging rwvmv ( vertical stress)
     izmax_inc=XC2dVTIRTM_izmax_inc;
     tsign=1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_isdep, XC2dVTIRTM_irdep, &src[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv1, rwvmv1 );

     tsign=-1;
     stat=XC2dVTIWaveprop_op( tsign, XC2dVTIRTM_irdep, XC2dVTIRTM_isdep, &dat[is*nx*nt], XC2dVTIRTM_veldtinvdx2, XC2dVTIRTM_epsf, XC2dVTIRTM_deltaf, izmax_inc, dummydat, dummydat, pwvmv2, rwvmv2 );

    for (it=0;it<nt/jtmv;it++)
       for (ix=2;ix<nx-2;ix++)
          for (iz=2;iz<nz-2;iz++)
          {
             idx=Wpzxloc(iz,ix);
             tmp=-1./12.*(pwvmv1[idx-2*nz+it*nx*nz]+pwvmv1[idx+2*nz+it*nx*nz])+4./3.*(pwvmv1[idx-nz+it*nx*nz]+pwvmv1[idx+nz+it*nx*nz])-2.5*pwvmv1[idx+it*nx*nz];
             tmp1=-1./12.*(rwvmv1[idx-2+it*nx*nz]+rwvmv1[idx+2+it*nx*nz])+4./3.*(rwvmv1[idx-1+it*nx*nz]+rwvmv1[idx+1+it*nx*nz])-2.5*rwvmv1[idx+it*nx*nz];
             imag[idx]+=tmp*pwvmv2[idx+nx*nz*it];
             imag[idx+1*nx*nz]+=tmp1*pwvmv2[idx+nx*nz*it];
             imag[idx+2*nx*nz]+=tmp*rwvmv2[idx+nx*nz*it];
             imag[idx+3*nx*nz]+=tmp1*rwvmv2[idx+nx*nz*it];
          } 
    if (stat==1) 
        Errmsg("Error in XC2dVTIWaveprop_op, program terminated!\n")   
  }  
  
  for (ix=2;ix<nx-2;ix++)
     for (iz=0;iz<nz;iz++)
     {
        idx=Wpzxloc(iz,ix);
        imag[idx]=imag[idx]/(dx*dx);
        imag[idx+1*nx*nz]=imag[idx+1*nx*nz]/(dx*dx);
        imag[idx+2*nx*nz]=imag[idx+2*nx*nz]/(dx*dx);
        imag[idx+3*nx*nz]=imag[idx+3*nx*nz]/(dx*dx);
     }
  return 0;
}
