#include "stdlib.h"
#include "math.h"
#include "string.h"
#include <sep_pars_external.h>
#include <seplib.h>
#include <sep3dc.h>
#include "fftw3.h"
#include "XC2dVTIRTM.c"

int main(int argc, char** argv)
{
    int ndim,i,n;
    float o,d;
    char temp_char[256];
    int   nt, nx, nz, ns;
    float ot, ox, oz;
    float dt, dx, dz;
    int   it, ix, iz, is;
    int   nxtape,nztape;
    int   needwvmv,jtmv,wvkernel;
    int   stat;
    float *vel, *eps, *delta, *src, *data, *sdep, *rdep, *imag;
    sep3d outfile; 
    time_t time1,time2;

    initpar(argc,argv);
    // read t,x,y from source file
   
    if(0!=sep_get_number_data_axes("in",&ndim))
       seperr("unable to get dimension from src!\n");
    
    for(i=1;i<=ndim;i++)
    {
       if(0!=sep_get_data_axis_par("in",&i,&n,&o,&d,temp_char))
       seperr("unable to get parameters from src!\n");
       if(i==1)
       {
          nt=n;
          dt=d;
          ot=o;
       }
       else if(i==2)
       {
          nx=n;
          dx=d;
          ox=o;
       }
       if (ndim==2) 
          ns=1; 
       else if(i==3)
          ns=n;
    }

    // read z info from vel
    if(0!=sep_get_number_data_axes("vel",&ndim))
       seperr("unable to get dimension from vel file!\n");
    for(i=1;i<=ndim;i++)
    {
       if(0!=sep_get_data_axis_par("vel",&i,&n,&o,&d,temp_char))
       seperr("unable to get parameters from vel file!\n");
       if(i==1)
       {
          nz=n;
          dz=d;
          oz=o;
       }
       else if(i==2)
       {
          if (nx!=n)
          Errmsg3("nx in vel %d different from nx %d in src, program terminated!\n",nx,n)
          if (dx!=d)
          Errmsg3("dx in vel %f different from dx %f in src, program terminated!\n",dx,d)
          if (ox!=o)
          Errmsg3("ox in vel %f different from ox %f in src, program terminated!\n",ox,o)
       }
    }

    // readin other parameters
    if (0==getch("nxtape","i", &nxtape)) nxtape=50;
    if (0==getch("nztape","i", &nztape)) nztape=50;
    if (0==getch("jtmv","i", &jtmv)) jtmv=10;
    if (0==getch("wvkernel","i", &wvkernel)) wvkernel=0;

    if (nx<=2*nxtape)
       Errmsg3("Error, nx %d less or equal than twice of padded x reigon %d \n",nx,nxtape);
    if (nz<=2*nztape)
       Errmsg3("Error, nz %d less or equal than twice of padded z reigon %d \n",nz,nztape);

    // allocate memories

    vel=(float*)malloc(nx*nz*sizeof(float));
    eps=(float*)malloc(nx*nz*sizeof(float));
    delta=(float*)malloc(nx*nz*sizeof(float));
    src=(float*)malloc(nx*nt*ns*sizeof(float));
    data=(float*)malloc(nx*nt*ns*sizeof(float));
    imag=(float*)malloc(nx*nz*4*sizeof(float));
    sdep=(float*)malloc(nx*ns*sizeof(float));
    rdep=(float*)malloc(nx*ns*sizeof(float));

    // read in various data files into arrays
    if ((nx*nt*ns*4)!=sreed("in",src,nx*nt*ns*4))
       Errmsg2("Error read in src file of size %d \n",nx*nt*ns*4);

    if ((nx*nt*ns*4)!=sreed("data",data,nx*nt*ns*4))
       Errmsg2("Error read in data file of size %d \n",nx*nt*ns*4);

    if ((nx*nz*4)!=sreed("vel",vel,nx*nz*4))
       Errmsg2("Error read in vel file of size %d \n", nx*nz*4);

    if ((nx*nz*4)!=sreed("eps",eps,nx*nz*4))
       Errmsg2("Error read in eps file of size %d \n", nx*nz*4);

    if ((nx*nz*4)!=sreed("delta",delta,nx*nz*4))
       Errmsg2("Error read in delta file of size %d \n", nx*nz*4);
 
    char file[80];
    if (0!=getch("sdep","s",file))
    {
       fprintf(stderr,"start reading sdep!!!\n");
       if ((nx*ns*4)!=sreed("sdep",sdep,nx*ns*4))
          seperr("Error read in rdep file\n");
    }
    else
    {
       for (i=0;i<nx*ns;i++)
          sdep[i]=2*dz;
    }

    if (0!=getch("rdep","s",file))
    {
       fprintf(stderr,"start reading rdep!!!\n");
       if ((nx*ns*4)!=sreed("rdep",rdep,nx*ns*4))
          seperr("Error read in rdep file\n");
    }
    else
    {
       for (i=0;i<nx*ns;i++)
          rdep[i]=2*dz;
    }

     
    // Initialize wave propagation 
    VerbMsg("start initialize XC2dVTIRTM_Init\n")
    stat=XC2dVTIRTM_Init( nt,nx,nz,ns,       \
                           ot,ox,oz,          \
                           dt,dx,dz,          \
                           nxtape,nztape, \
                           sdep, rdep, vel, eps, delta,  \
                           jtmv );

    if (stat==1)
       seperr("Error in XC2dVTIRTM_Init,program terminated!\n");
    
    // run wave propagation 
    VerbMsg("start modeling in XC2dVTIRTM_op \n")
    time1=time(NULL);
    int tsign=1;
    if (wvkernel==1)
       stat=XC2dVTIRTM_wvkernel1(src,data, imag);
    else if (wvkernel==11)
       stat=XC2dVTIRTM_wvkernel11(src,data, imag);
    else if (wvkernel==2)
       stat=XC2dVTIRTM_wvkernel2(src,data, imag);
    else if (wvkernel==3)
       stat=XC2dVTIRTM_wvkernel3(src,data, imag);
    else if (wvkernel==0)
       stat=XC2dVTIRTM_op(src,data, imag);
    time2=time(NULL);

    fprintf(stderr,"used time for 1 shot is %ld seconds\n", time2-time1);
    if (stat==1)
       seperr("Error in XC2dVTIRTM_op,program terminated!\n");

    //output recorded data
    //sensitivity kernel of dm/dp, dm/dr, dx/dp, dx/dr
    VerbMsg("start output imag \n")
    ndim=3;
    if(0!=init_sep3d_par(&outfile,"OUTPUT","FLOAT","REGULAR",ndim,0))
       seperr("unable to write outfile!\n");
    outfile.o[0]=oz;
    outfile.d[0]=dz;
    outfile.n[0]=nz;
    outfile.o[1]=ox;
    outfile.d[1]=dx;
    outfile.n[1]=nx;
    outfile.o[2]=0;
    outfile.d[2]=1;
    outfile.n[2]=4;
    sep3d_set_sep3d(&outfile);
    if(0!=sep3dc_write_description("imag",&outfile))
       seperr("unable to output parameters for imag\n");

    if( (nz*nx*4*4)!=srite("imag",imag,4*4*nz*nx) )
       seperr("trouble writing out imag\n");

    return 0;
}
